@@ -152,6 +152,11 @@ class ServerListSecurityGroups(task_manager.Task):
         return client.nova_client.servers.list_security_group(**self.args)
 
 
+class ServerConsoleGet(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.servers.get_console_output(**self.args)
+
+
 class ServerGet(task_manager.Task):
     def main(self, client):
         return client.nova_client.servers.get(**self.args)
@@ -404,6 +409,21 @@ class VolumeSnapshotList(task_manager.Task):
         return client.cinder_client.volume_snapshots.list(**self.args)
 
 
+class VolumeBackupList(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.backups.list(**self.args)
+
+
+class VolumeBackupCreate(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.backups.create(**self.args)
+
+
+class VolumeBackupDelete(task_manager.Task):
+    def main(self, client):
+        return client.cinder_client.backups.delete(**self.args)
+
+
 class VolumeSnapshotDelete(task_manager.Task):
     def main(self, client):
         return client.cinder_client.volume_snapshots.delete(**self.args)
@@ -411,7 +431,7 @@ class VolumeSnapshotDelete(task_manager.Task):
 
 class NeutronSecurityGroupList(task_manager.Task):
     def main(self, client):
-        return client.neutron_client.list_security_groups()
+        return client.neutron_client.list_security_groups(**self.args)
 
 
 class NeutronSecurityGroupCreate(task_manager.Task):
@@ -441,7 +461,7 @@ class NeutronSecurityGroupRuleDelete(task_manager.Task):
 
 class NovaSecurityGroupList(task_manager.Task):
     def main(self, client):
-        return client.nova_client.security_groups.list()
+        return client.nova_client.security_groups.list(**self.args)
 
 
 class NovaSecurityGroupCreate(task_manager.Task):
