@@ -21,7 +21,6 @@ Tests Floating IP resource methods for Neutron
 
 import mock
 from mock import patch
-import os_client_config
 
 from neutronclient.common import exceptions as n_exc
 
@@ -138,10 +137,6 @@ class TestFloatingIP(base.TestCase):
 
     def setUp(self):
         super(TestFloatingIP, self).setUp()
-        # floating_ip_source='neutron' is default for OpenStackCloud()
-        config = os_client_config.OpenStackConfig()
-        self.client = OpenStackCloud(
-            cloud_config=config.get_one_cloud(validate=False))
 
         self.fake_server = meta.obj_to_dict(
             fakes.FakeServer(
@@ -152,23 +147,52 @@ class TestFloatingIP(base.TestCase):
                     u'version': 4,
                     u'OS-EXT-IPS-MAC:mac_addr':
                     u'fa:16:3e:ae:7d:42'}]}))
-        self.floating_ip = _utils.normalize_neutron_floating_ips(
+        self.floating_ip = self.cloud._normalize_floating_ips(
             self.mock_floating_ip_list_rep['floatingips'])[0]
 
+    def test_float_no_status(self):
+        floating_ips = [
+            {
+                'fixed_ip_address': '10.0.0.4',
+                'floating_ip_address': '172.24.4.229',
+                'floating_network_id': 'my-network-id',
+                'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda8',
+                'port_id': None,
+                'router_id': None,
+                'tenant_id': '4969c491a3c74ee4af974e6d800c62df'
+            }
+        ]
+        normalized = self.cloud._normalize_floating_ips(floating_ips)
+        self.assertEqual('UNKNOWN', normalized[0]['status'])
+
     @patch.object(OpenStackCloud, 'neutron_client')
-    @patch.object(OpenStackCloud, 'has_service')
+    @patch.object(OpenStackCloud, 'has_service', return_value=True)
     def test_list_floating_ips(self, mock_has_service, mock_neutron_client):
-        mock_has_service.return_value = True
         mock_neutron_client.list_floatingips.return_value = \
             self.mock_floating_ip_list_rep
 
-        floating_ips = self.client.list_floating_ips()
+        floating_ips = self.cloud.list_floating_ips()
 
         mock_neutron_client.list_floatingips.assert_called_with()
         self.assertIsInstance(floating_ips, list)
         self.assertAreInstances(floating_ips, dict)
         self.assertEqual(2, len(floating_ips))
 
+    @patch.object(OpenStackCloud, 'neutron_client')
+    @patch.object(OpenStackCloud, 'has_service', return_value=True)
+    def test_list_floating_ips_with_filters(self, mock_has_service,
+                                            mock_neutron_client):
+        mock_neutron_client.list_floatingips.side_effect = \
+            exc.OpenStackCloudURINotFound("")
+
+        try:
+            self.cloud.list_floating_ips(filters={'Foo': 42})
+        except exc.OpenStackCloudException as e:
+            self.assertIsInstance(
+                e.inner_exception[1], exc.OpenStackCloudURINotFound)
+
+        mock_neutron_client.list_floatingips.assert_called_once_with(Foo=42)
+
     @patch.object(OpenStackCloud, 'neutron_client')
     @patch.object(OpenStackCloud, 'has_service')
     def test_search_floating_ips(self, mock_has_service, mock_neutron_client):
@@ -176,10 +200,10 @@ class TestFloatingIP(base.TestCase):
         mock_neutron_client.list_floatingips.return_value = \
             self.mock_floating_ip_list_rep
 
-        floating_ips = self.client.search_floating_ips(
+        floating_ips = self.cloud.search_floating_ips(
             filters={'attached': False})
 
-        mock_neutron_client.list_floatingips.assert_called_with()
+        mock_neutron_client.list_floatingips.assert_called_with(attached=False)
         self.assertIsInstance(floating_ips, list)
         self.assertAreInstances(floating_ips, dict)
         self.assertEqual(1, len(floating_ips))
@@ -191,12 +215,21 @@ class TestFloatingIP(base.TestCase):
         mock_neutron_client.list_floatingips.return_value = \
             self.mock_floating_ip_list_rep
 
-        floating_ip = self.client.get_floating_ip(
+        floating_ip = self.cloud.get_floating_ip(
             id='2f245a7b-796b-4f26-9cf9-9e82d248fda7')
 
         mock_neutron_client.list_floatingips.assert_called_with()
         self.assertIsInstance(floating_ip, dict)
         self.assertEqual('203.0.113.29', floating_ip['floating_ip_address'])
+        self.assertEqual(
+            self.mock_floating_ip_list_rep['floatingips'][0]['tenant_id'],
+            floating_ip['project_id']
+        )
+        self.assertEqual(
+            self.mock_floating_ip_list_rep['floatingips'][0]['tenant_id'],
+            floating_ip['tenant_id']
+        )
+        self.assertIn('location', floating_ip)
 
     @patch.object(OpenStackCloud, 'neutron_client')
     @patch.object(OpenStackCloud, 'has_service')
@@ -206,7 +239,7 @@ class TestFloatingIP(base.TestCase):
         mock_neutron_client.list_floatingips.return_value = \
             self.mock_floating_ip_list_rep
 
-        floating_ip = self.client.get_floating_ip(id='non-existent')
+        floating_ip = self.cloud.get_floating_ip(id='non-existent')
 
         self.assertIsNone(floating_ip)
 
@@ -220,7 +253,7 @@ class TestFloatingIP(base.TestCase):
         mock_neutron_client.create_floatingip.return_value = \
             self.mock_floating_ip_new_rep
 
-        ip = self.client.create_floating_ip(network='my-network')
+        ip = self.cloud.create_floating_ip(network='my-network')
 
         mock_neutron_client.create_floatingip.assert_called_with(
             body={'floatingip': {'floating_network_id': 'my-network-id'}}
@@ -241,7 +274,7 @@ class TestFloatingIP(base.TestCase):
 
         self.assertRaises(
             exc.OpenStackCloudException,
-            self.client.create_floating_ip,
+            self.cloud.create_floating_ip,
             network='my-network', port='ce705c24-c1ef-408a-bda3-7bbd946164ab')
 
     @patch.object(OpenStackCloud, 'neutron_client')
@@ -254,7 +287,7 @@ class TestFloatingIP(base.TestCase):
         mock_neutron_client.create_floatingip.return_value = \
             self.mock_floating_ip_port_rep
 
-        ip = self.client.create_floating_ip(
+        ip = self.cloud.create_floating_ip(
             network='my-network', port='ce705c24-c1ef-408a-bda3-7bbd946164ab')
 
         mock_neutron_client.create_floatingip.assert_called_with(
@@ -267,33 +300,10 @@ class TestFloatingIP(base.TestCase):
             self.mock_floating_ip_new_rep['floatingip']['floating_ip_address'],
             ip['floating_ip_address'])
 
-    @patch.object(_utils, 'normalize_neutron_floating_ips')
-    @patch.object(OpenStackCloud, '_neutron_available_floating_ips')
-    @patch.object(OpenStackCloud, 'has_service')
-    @patch.object(OpenStackCloud, 'keystone_session')
-    def test_available_floating_ip_neutron(self,
-                                           mock_keystone,
-                                           mock_has_service,
-                                           mock__neutron_call,
-                                           mock_normalize):
-        """
-        Test the correct path is taken when using neutron.
-        """
-        # force neutron path
-        mock_has_service.return_value = True
-        mock__neutron_call.return_value = []
-
-        self.client.available_floating_ip(network='netname')
-
-        mock_has_service.assert_called_once_with('network')
-        mock__neutron_call.assert_called_once_with(network='netname',
-                                                   server=None)
-        mock_normalize.assert_called_once_with([])
-
     @patch.object(_utils, '_filter_list')
     @patch.object(OpenStackCloud, '_neutron_create_floating_ip')
     @patch.object(OpenStackCloud, '_neutron_list_floating_ips')
-    @patch.object(OpenStackCloud, 'get_external_networks')
+    @patch.object(OpenStackCloud, 'get_external_ipv4_floating_networks')
     @patch.object(OpenStackCloud, 'keystone_session')
     def test__neutron_available_floating_ips(
             self,
@@ -311,11 +321,11 @@ class TestFloatingIP(base.TestCase):
         mock__filter_list.return_value = []
 
         # Test if first network is selected if no network is given
-        self.client._neutron_available_floating_ips()
+        self.cloud._neutron_available_floating_ips()
 
         mock_keystone_session.get_project_id.assert_called_once_with()
         mock_get_ext_nets.assert_called_once_with()
-        mock__neutron_list_fips.assert_called_once_with()
+        mock__neutron_list_fips.assert_called_once_with(None)
         mock__filter_list.assert_called_once_with(
             [], name_or_id=None,
             filters={'port_id': None,
@@ -323,14 +333,14 @@ class TestFloatingIP(base.TestCase):
                      'tenant_id': 'proj-id'}
         )
         mock__neutron_create_fip.assert_called_once_with(
-            network_name_or_id=self.mock_get_network_rep['id'],
+            network_id=self.mock_get_network_rep['id'],
             server=None
         )
 
     @patch.object(_utils, '_filter_list')
     @patch.object(OpenStackCloud, '_neutron_create_floating_ip')
     @patch.object(OpenStackCloud, '_neutron_list_floating_ips')
-    @patch.object(OpenStackCloud, 'get_external_networks')
+    @patch.object(OpenStackCloud, 'get_external_ipv4_floating_networks')
     @patch.object(OpenStackCloud, 'keystone_session')
     def test__neutron_available_floating_ips_network(
             self,
@@ -347,13 +357,13 @@ class TestFloatingIP(base.TestCase):
         mock__neutron_list_fips.return_value = []
         mock__filter_list.return_value = []
 
-        self.client._neutron_available_floating_ips(
+        self.cloud._neutron_available_floating_ips(
             network=self.mock_get_network_rep['name']
         )
 
         mock_keystone_session.get_project_id.assert_called_once_with()
         mock_get_ext_nets.assert_called_once_with()
-        mock__neutron_list_fips.assert_called_once_with()
+        mock__neutron_list_fips.assert_called_once_with(None)
         mock__filter_list.assert_called_once_with(
             [], name_or_id=None,
             filters={'port_id': None,
@@ -361,11 +371,11 @@ class TestFloatingIP(base.TestCase):
                      'tenant_id': 'proj-id'}
         )
         mock__neutron_create_fip.assert_called_once_with(
-            network_name_or_id=self.mock_get_network_rep['id'],
+            network_id=self.mock_get_network_rep['id'],
             server=None
         )
 
-    @patch.object(OpenStackCloud, 'get_external_networks')
+    @patch.object(OpenStackCloud, 'get_external_ipv4_networks')
     @patch.object(OpenStackCloud, 'keystone_session')
     def test__neutron_available_floating_ips_invalid_network(
             self,
@@ -377,7 +387,7 @@ class TestFloatingIP(base.TestCase):
         mock_keystone_session.get_project_id.return_value = 'proj-id'
         mock_get_ext_nets.return_value = []
         self.assertRaises(exc.OpenStackCloudException,
-                          self.client._neutron_available_floating_ips,
+                          self.cloud._neutron_available_floating_ips,
                           network='INVALID')
 
     @patch.object(OpenStackCloud, 'nova_client')
@@ -392,14 +402,14 @@ class TestFloatingIP(base.TestCase):
             mock_keystone_session,
             mock_nova_client):
         mock_has_service.return_value = True
-        fip = _utils.normalize_neutron_floating_ips(
+        fip = self.cloud._normalize_floating_ips(
             self.mock_floating_ip_list_rep['floatingips'])[0]
         mock__neutron_create_floating_ip.return_value = fip
         mock_keystone_session.get_project_id.return_value = \
             '4969c491a3c74ee4af974e6d800c62df'
         fake_server = meta.obj_to_dict(fakes.FakeServer('1234', '', 'ACTIVE'))
 
-        self.client.add_ips_to_server(
+        self.cloud.add_ips_to_server(
             fake_server, ip_pool='my-network', reuse=False)
 
         mock__neutron_create_floating_ip.assert_called_once_with(
@@ -426,7 +436,7 @@ class TestFloatingIP(base.TestCase):
         mock_keystone_session.get_project_id.return_value = \
             '4969c491a3c74ee4af974e6d800c62df'
 
-        ip = self.client.available_floating_ip(network='my-network')
+        ip = self.cloud.available_floating_ip(network='my-network')
 
         self.assertEqual(
             self.mock_floating_ip_new_rep['floatingip']['floating_ip_address'],
@@ -441,12 +451,13 @@ class TestFloatingIP(base.TestCase):
         fake_fip = {
             'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda7',
             'floating_ip_address': '172.99.106.167',
+            'status': 'ACTIVE',
         }
 
         mock_get_floating_ip.side_effect = [fake_fip, fake_fip, None]
         mock_neutron_client.delete_floatingip.return_value = None
 
-        ret = self.client.delete_floating_ip(
+        ret = self.cloud.delete_floating_ip(
             floating_ip_id='2f245a7b-796b-4f26-9cf9-9e82d248fda7',
             retry=2)
 
@@ -456,6 +467,36 @@ class TestFloatingIP(base.TestCase):
         self.assertEqual(mock_get_floating_ip.call_count, 3)
         self.assertTrue(ret)
 
+    @patch.object(OpenStackCloud, 'get_floating_ip')
+    @patch.object(OpenStackCloud, 'neutron_client')
+    @patch.object(OpenStackCloud, 'has_service')
+    def test_delete_floating_ip_existing_down(
+            self, mock_has_service, mock_neutron_client, mock_get_floating_ip):
+        mock_has_service.return_value = True
+        fake_fip = {
+            'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda7',
+            'floating_ip_address': '172.99.106.167',
+            'status': 'ACTIVE',
+        }
+        down_fip = {
+            'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda7',
+            'floating_ip_address': '172.99.106.167',
+            'status': 'DOWN',
+        }
+
+        mock_get_floating_ip.side_effect = [fake_fip, down_fip, None]
+        mock_neutron_client.delete_floatingip.return_value = None
+
+        ret = self.cloud.delete_floating_ip(
+            floating_ip_id='2f245a7b-796b-4f26-9cf9-9e82d248fda7',
+            retry=2)
+
+        mock_neutron_client.delete_floatingip.assert_called_with(
+            floatingip='2f245a7b-796b-4f26-9cf9-9e82d248fda7'
+        )
+        self.assertEqual(mock_get_floating_ip.call_count, 2)
+        self.assertTrue(ret)
+
     @patch.object(OpenStackCloud, 'get_floating_ip')
     @patch.object(OpenStackCloud, 'neutron_client')
     @patch.object(OpenStackCloud, 'has_service')
@@ -465,6 +506,7 @@ class TestFloatingIP(base.TestCase):
         fake_fip = {
             'id': '2f245a7b-796b-4f26-9cf9-9e82d248fda7',
             'floating_ip_address': '172.99.106.167',
+            'status': 'ACTIVE',
         }
 
         mock_get_floating_ip.side_effect = [fake_fip, fake_fip, fake_fip]
@@ -472,7 +514,7 @@ class TestFloatingIP(base.TestCase):
 
         self.assertRaises(
             exc.OpenStackCloudException,
-            self.client.delete_floating_ip,
+            self.cloud.delete_floating_ip,
             floating_ip_id='2f245a7b-796b-4f26-9cf9-9e82d248fda7',
             retry=2)
 
@@ -489,7 +531,7 @@ class TestFloatingIP(base.TestCase):
         mock_neutron_client.delete_floatingip.side_effect = \
             n_exc.NotFound()
 
-        ret = self.client.delete_floating_ip(
+        ret = self.cloud.delete_floating_ip(
             floating_ip_id='a-wild-id-appears')
 
         self.assertFalse(ret)
@@ -506,7 +548,7 @@ class TestFloatingIP(base.TestCase):
         mock_neutron_client.list_floatingips.return_value = \
             self.mock_floating_ip_list_rep
 
-        self.client._attach_ip_to_server(
+        self.cloud._attach_ip_to_server(
             server=self.fake_server,
             floating_ip=self.floating_ip)
 
@@ -522,22 +564,22 @@ class TestFloatingIP(base.TestCase):
         )
 
     @patch.object(OpenStackCloud, 'delete_floating_ip')
-    @patch.object(OpenStackCloud, 'get_server_by_id')
+    @patch.object(OpenStackCloud, 'get_server')
     @patch.object(OpenStackCloud, 'create_floating_ip')
     @patch.object(OpenStackCloud, 'has_service')
     def test_add_ip_refresh_timeout(
             self, mock_has_service, mock_create_floating_ip,
-            mock_get_server_by_id, mock_delete_floating_ip):
+            mock_get_server, mock_delete_floating_ip):
         mock_has_service.return_value = True
 
         mock_create_floating_ip.return_value = self.floating_ip
-        mock_get_server_by_id.return_value = self.fake_server
+        mock_get_server.return_value = self.fake_server
 
         self.assertRaises(
             exc.OpenStackCloudTimeout,
-            self.client._add_auto_ip,
+            self.cloud._add_auto_ip,
             server=self.fake_server,
-            wait=True, timeout=2,
+            wait=True, timeout=0.01,
             reuse=False)
 
         mock_delete_floating_ip.assert_called_once_with(
@@ -551,10 +593,10 @@ class TestFloatingIP(base.TestCase):
             mock_get_floating_ip):
         mock_has_service.return_value = True
         mock_get_floating_ip.return_value = \
-            _utils.normalize_neutron_floating_ips(
+            self.cloud._normalize_floating_ips(
                 self.mock_floating_ip_list_rep['floatingips'])[0]
 
-        self.client.detach_ip_from_server(
+        self.cloud.detach_ip_from_server(
             server_id='server-id',
             floating_ip_id='2f245a7b-796b-4f26-9cf9-9e82d248fda7')
 
@@ -575,11 +617,11 @@ class TestFloatingIP(base.TestCase):
             mock_attach_ip_to_server):
         mock_has_service.return_value = True
         mock_available_floating_ip.return_value = \
-            _utils.normalize_neutron_floating_ips([
+            self.cloud._normalize_floating_ips([
                 self.mock_floating_ip_new_rep['floatingip']])[0]
         mock_attach_ip_to_server.return_value = self.fake_server
 
-        server = self.client._add_ip_from_pool(
+        server = self.cloud._add_ip_from_pool(
             server=self.fake_server,
             network='network-name',
             fixed_address='192.0.2.129')
@@ -613,23 +655,23 @@ class TestFloatingIP(base.TestCase):
 
         mock_list_floating_ips.return_value = floating_ips
 
-        self.client.delete_unattached_floating_ips()
+        self.cloud.delete_unattached_floating_ips()
 
         mock_delete_floating_ip.assert_called_once_with(
             floating_ip_id='this-is-a-floating-ip-id', retry=1)
 
     @patch.object(OpenStackCloud, '_submit_create_fip')
-    @patch.object(OpenStackCloud, '_get_free_fixed_port')
-    @patch.object(OpenStackCloud, 'get_external_networks')
+    @patch.object(OpenStackCloud, '_nat_destination_port')
+    @patch.object(OpenStackCloud, 'get_external_ipv4_networks')
     def test_create_floating_ip_no_port(
-            self, mock_get_ext_nets, mock_get_free_fixed_port,
+            self, mock_get_ext_nets, mock_nat_destination_port,
             mock_submit_create_fip):
         fake_port = dict(id='port-id')
         mock_get_ext_nets.return_value = [self.mock_get_network_rep]
-        mock_get_free_fixed_port.return_value = (fake_port, '10.0.0.2')
+        mock_nat_destination_port.return_value = (fake_port, '10.0.0.2')
         mock_submit_create_fip.return_value = dict(port_id=None)
 
         self.assertRaises(
             exc.OpenStackCloudException,
-            self.client._neutron_create_floating_ip,
+            self.cloud._neutron_create_floating_ip,
             server=dict(id='some-server'))
